@@ -23,6 +23,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 
 #include <glib.h>
@@ -44,19 +45,21 @@ main (int argc, char **argv)
         GError        *error;
         gboolean       ret;
         char          *location = NULL;
+        char          *background_color = NULL;
+        gboolean       sort_images = FALSE;
         GOptionEntry  entries [] = {
                 { "location", 0, 0, G_OPTION_ARG_STRING, &location,
                   N_("Location to get images from"), N_("PATH") },
+                { "background-color", 0, 0, G_OPTION_ARG_STRING, &background_color,
+                  N_("Color to use for images background"), N_("#abcdef") },
+                { "sort-images", 0, 0, G_OPTION_ARG_NONE, &sort_images,
+                  N_("Do not randomize pictures from location"), NULL },
                 { NULL }
         };
 
-#ifdef ENABLE_NLS
         bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-# ifdef HAVE_BIND_TEXTDOMAIN_CODESET
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-# endif
         textdomain (GETTEXT_PACKAGE);
-#endif
 
         error = NULL;
 
@@ -96,6 +99,14 @@ main (int argc, char **argv)
                 g_object_set (engine, "images-location", location, NULL);
         }
 
+        if (sort_images) {
+                g_object_set (engine, "sort-images", sort_images, NULL);
+        }
+
+        if (background_color != NULL) {
+                g_object_set (engine, "background-color", background_color, NULL);
+        }
+
         gtk_container_add (GTK_CONTAINER (window), GTK_WIDGET (engine));
 
         gtk_widget_show (GTK_WIDGET (engine));
